%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MATLAB-Demo: Distortion of a wav-signal
% by Meinard Mueller, 19.02.2004
%
% Zero phase
% Randomly distorted phase
% Distorted amplitude
% Thresholded amplitude
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear;
close all;

% [file,fs,nbits] = wavread('BB_F_Kadenz_stacc_a.wav');
% [file,fs,nbits] = wavread('Sys_tonleiter_CDur_mittel_BB_F.wav');
[file,fs,nbits] =  wavread('fk_zuerich_11025_stereo.wav');
%sound(file,fs)

% Make "mono" from "stereo"
file = file(:,1);

% Take FFT of length which is power of two above length(file) 
fft_length = 2^ceil(log2(length(file)));
file = [file;zeros(fft_length-size(file,1),1)];
file_fft = fft(file,fft_length);

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% signal and fft
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure;
subplot(4,1,1);
plot(real(file));
title('Signal f and Fourier transform');
xlabel('real(f)')
axis tight;
subplot(4,1,2);
plot(abs(file_fft));
xlabel('abs(FFT(f))')
axis tight;
subplot(4,1,3);
plot(angle(file_fft));
xlabel('phase(FFT(f))')
axis tight;
subplot(4,1,4);
plot(unwrap(angle(file_fft)));
xlabel('unwraped(phase(FFT(f)))')
axis tight;
%sound(file,fs)

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% zero phase
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
file_fft_zero_phase = file_fft.*(exp(-i*angle(file_fft)));
file_zero_phase = ifft(file_fft_zero_phase,fft_length);
figure;
subplot(4,1,1);
plot(real(file));
title('Setting the phase of f to zero obtaining g');
xlabel('real(f)')
axis tight;
%title('real(f), phase(f), real(g) with g = modified f with zero phase, phase(g)');
subplot(4,1,2);
plot(unwrap(angle(file_fft)));
xlabel('phase(FFT(f))')
axis tight;
subplot(4,1,3);
plot(unwrap(angle(file_fft_zero_phase)));
axis tight;
xlabel('phase(FFT(g))')
subplot(4,1,4);
plot(real(file_zero_phase));
xlabel('real(g)')
axis tight;
%sound(real(file_zero_phase),fs);

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% distorted noise phase
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
noise = 3*(rand(fft_length,1)-0.5);
file_fft_dis_phase = file_fft.*(exp(-i*noise));
file_dis_phase = ifft(file_fft_dis_phase,fft_length);

figure;
subplot(4,1,1);
plot(real(file));
title('Distorting the phase of f by random noise obtaining g');
xlabel('real(f)')
axis tight;
subplot(4,1,2);
plot(unwrap(angle(file_fft)));
xlabel('phase(f)')
axis tight;
subplot(4,1,3);
plot(unwrap(angle(file_fft_dis_phase)));
xlabel('phase(g)')
axis tight;
subplot(4,1,4);
plot(real(file_dis_phase));
xlabel('real(g)')
axis tight;
%sound(real(file_dis_phase),fs);

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% distorted noise amplitude
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

noise = 25*(rand(fft_length,1)-0.5);
file_fft_dis_amp = file_fft+noise;
file_dis_amp = ifft(file_fft_dis_amp,fft_length);

figure;
subplot(4,1,1);
plot(real(file));
xlabel('real(f)')
axis tight;
title('Distorting the amplitude of f by adding random noise obtaining g');
subplot(4,1,2);
plot(abs(file_fft));
xlabel('abs(fft(f))')
axis tight;
subplot(4,1,3);
plot(abs(file_fft_dis_amp));
xlabel('abs(fft(g))')
axis tight;
subplot(4,1,4);
plot(real(file_dis_amp));
xlabel('real(g)')
axis tight;
%sound(real(file_dis_amp),fs);

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% thresholded amplitude
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
num = 1000;                       %number of largest Fourier coefficients
[s,index] = sort(abs(file_fft));
file_fft_thresh = zeros(fft_length,1);
file_fft_thresh(index(fft_length-num:fft_length))=file_fft(index(fft_length-num:fft_length));
file_thresh = ifft(file_fft_thresh,fft_length);

figure;
subplot(4,1,1);
plot(real(file));
title('Picking only largest Fourier coefficients of f obtaining g');
xlabel('real(f)')
axis tight;
subplot(4,1,2);
plot(abs(file_fft));
xlabel('abs(fft(f))')
axis tight;
subplot(4,1,3);
plot(abs(file_fft_thresh));
xlabel('abs(fft(g))')
axis tight;
subplot(4,1,4);
plot(real(file_thresh));
xlabel('real(g)')
axis tight;
%sound(real(file_thresh),fs);

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% constant amplitude
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
file_fft_const_amp = 0.2*exp(i*angle(file_fft));
file_const_amp = ifft(file_fft_const_amp,fft_length);

figure;
subplot(4,1,1);
plot(real(file));
title('Setting all Fourier cofficients of f to contant obtaining g');
xlabel('real(f)')
axis tight;
subplot(4,1,2);
plot(abs(file_fft));
xlabel('abs(fft(f))')
axis tight;
subplot(4,1,3);
plot(abs(file_fft_const_amp));
xlabel('abs(fft(g))')
axis tight;
subplot(4,1,4);
plot(real(file_const_amp));
xlabel('real(g)')
axis tight;
%sound(real(file_dis_amp),fs);
