%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MATLAB-Demo: Wav-Wav-Synchronization
% by Meinard Mueller, 19.02.2004
%
% Aufruf der Funktion: [file,fs,file2,fs2]=DEMO_sync_wav_wav;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [file,fs,file2,fs2]=DEMO_sync_wav_wav

clear;
close all;

window_length = 2048;
window = hanning(window_length);
overlap = fix(length(window)/2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Loading and normalizing wav-files to fs = 22050
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[file,fs,nbits] =  wavread('BB_F_Kadenz_portato_a.wav');
[file2,fs2,nbits2] =  wavread('BB_F_Kadenz_stacc_a.wav');

%[file,fs,nbits] =  wavread('Sys_tonleiter1b_midi_helga.wav');
%[file2,fs2,nbits2] =  wavread('Sys_tonleiter1c_midi_helga.wav');

%[file,fs,nbits] =  wavread('BM02A_meinard.wav');
%[file2,fs2,nbits2] =  wavread('BM02A.wav');


file = file(:,1);
file2 = file2(:,1);

if fs == 44100
    file = resample(file,1,2);
    fs = 22050;
    end
if fs2 == 44100 
    file2 = resample(file2,1,2);
    fs2 = 22050;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Computing and representing spectograms
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[B,f,t]=mm_specgram(file,fs,window_length,window,overlap,1);
[B2,f2,t2]=mm_specgram(file2,fs2,window_length,window,overlap,2);
n = length(t);
m = length(t2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Global score matrix for matching i-th feature with j-th feature
%  - i-th feature consists of the first spectral coefficients 
%    determined by freq_select
%  - score is absolute value cos-measure in case it is above threshold,
%    otherwise score is 0
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c = zeros(n,m);
freq_select = [1:256];
for i=1:n
    for j=1:m
        c(i,j)=abs(cos_measure(B(freq_select,i),B2(freq_select,j)));
      end
end

threshold = 0.5;
c = c.*(c>threshold);
figure;
mesh(c);
title('Local score matrix c(i,j)');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Computing optimal match by dynamic programming
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
D = zeros(n+1,m+1);
%Attention: MATLAB indexing begins always with index 1
for i=1:n
    for j=1:m
           D(i+1,j+1)=max([D(i,j+1), D(i+1,j), D(i,j)+c(i,j)]);
       end
end

match = zeros(max(n,m),2);
k=0;
while ((i>0) & (j>0))
    if D(i+1,j+1)==D(i+1,j)
        j=j-1;
    elseif D(i+1,j+1)==D(i,j+1)
        i=i-1;
    else
        k=k+1;
        match(k,:)=[i j];
        j=j-1;
        i=i-1;
    end
end
match = match([1:k],:);
match = flipud(match);

figure;
mesh(D);
title('Global score matrix D(i,j)');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Visualization of Match 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure;
ytrans = -0.5;
plot(file); 
hold; 
plot(file2+ytrans);
axis([1,max(length(file),length(file2)),min(file2)+ytrans,max(file)]);
title('Matching two signals');
stepsize=length(window)-overlap;
for i=1:length(match)
    handle=line([match(i,1)*stepsize, match(i,2)*stepsize],[0,ytrans]);
    set(handle,'Color','red');
end
    


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Computing and representing spectograms (windowed Fourier transform)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [B,f,t]=mm_specgram(file,fs,window_length,window,overlap,figure_number)

[B,f,t]=specgram(file,window_length,fs,window,overlap);
figure(figure_number);
axes('Position',[0.1,0.8,0.8,0.1])
plot(file)
title('Signal f and spectogram') 
axis([1,length(file),min(file),max(file)])
axes('Position',[0.1,0.1,0.8,0.63])
max_freq=length(f);
handle=image(log(0.01+   abs(B((1:max_freq),:))   )); 
set(handle,'Cdatamapping','scaled')
colormap('hot')
handle=gca;
set(handle,'YDir','normal');
xlabel('Time t')
ylabel('Frequency \omega')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Local score function
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function c = cos_measure(v,w)
    norm_v = norm(v);
    norm_w = norm(w);
    if ((norm_v == 0) | (norm_w == 0))
        c = 0;
    else
        c = (v'*w)/(norm_v*norm_w);
    end
